package console

import (
	"bufio"
	"bytes"
	"context"
	"doc-service/domain/entity/serversitemicros/docservercallback"
	"doc-service/domain/entity/serversitemicros/docserverproject"
	"doc-service/infra/db"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/nacos-group/nacos-sdk-go/v2/common/logger"
	"gitlab.galaxy-immi.com/Backend-group/go-com/third/log"
	"io"
	"net/http"
	"os"
	"strconv"
	"strings"
	"time"
)

func recoverPanicCallBack() {
	/*if r := recover(); r != nil {
		logger.Error("执行请求回调异常", r)
	}*/
}

// 执行回调
func DoCallback() {
	defer recoverPanicCallBack()
	fmt.Println("开始请求回调...")
	ctxCall := context.Background()
	tickerCallback := time.NewTicker(10 * time.Second)
	defer tickerCallback.Stop()

	go func() {
		for {
			select {
			case <-tickerCallback.C:
				//执行转换
				i := callbackData(ctxCall)
				if i == 0 {
					fmt.Println("暂无回调数据: " + time.Now().Format("2006-01-02 15:04:05"))
				}
			}
		}
	}()

	// 监听命令行输入
	reader := bufio.NewReader(os.Stdin)
	for {
		text, _ := reader.ReadString('\n')
		text = strings.TrimSpace(text)

		if text == "exit" {
			fmt.Println("Exiting process...")
			os.Exit(0)
		}
	}
}

func callbackData(ctx context.Context) int8 {
	defer recoverPanicCallBack()
	//列表
	callBackData, err := db.GetServerSiteMicrosDB().DocServerProject.Query().Where(
		docserverproject.StatusEQ(2),
	).All(ctx)

	if err != nil {
		return 0
	}

	if len(callBackData) <= 0 {
		return 0
	}

	for _, item := range callBackData {
		//执行转换
		res1, msg := callBackUrl(ctx, item.ID)
		var status int32
		if res1 {
			status = 3
		} else {
			status = 4
		}
		_, errUpdate := db.GetServerSiteMicrosDB().DocServerProject.UpdateOneID(item.ID).SetStatus(status).SetCallbackRes(msg).Save(ctx)
		if errUpdate != nil {
			log.Info("回调请求修改状态出错" + errUpdate.Error())
			continue
		}
		log.Info("请求回调完成: " + string(item.ID))
		fmt.Printf("请求回调完成:   %d\n", item.ID)
	}

	return 1
}

// Data 结构体定义
type BackData struct {
	ID     int    `json:"id"`
	Msg    string `json:"msg"`
	Status int8   `json:"status"`
	Url    string `json:"url"`
}

type CallBackData struct {
	ID int
}

// 执行请求
func callBackUrl(c context.Context, projectId int32) (bool, string) {
	defer recoverPanicCallBack()
	callInfo, callErr := db.GetServerSiteMicrosDB().DocServerCallback.Query().Where(
		docservercallback.ServerProjectID(projectId),
	).First(c)

	var msg string
	var res bool
	res = true
	msg = "success"
	if callErr != nil {
		logger.Error("获取回调单条出错", callErr.Error())
		msg = callErr.Error()
		res = false
	} else {
		if callInfo == nil {
			msg = "回调信息为空"
			res = false
		}
		if callInfo.CallbackURL == "" {
			msg = "回调地址为空"
			res = false
		}
	}

	projectInfo, _ := db.GetServerSiteMicrosDB().DocServerProject.Query().Where(
		docserverproject.IDEQ(projectId),
	).First(c)
	logger.Info(projectInfo)
	if res {
		//回调
		if callInfo.CallbackType == 0 {

			// 创建一个 Data 结构体实例
			data := &BackData{}
			id := 0

			if callInfo.CallbackData != "" {
				backData := CallBackData{}
				body := []byte(callInfo.CallbackData)
				_ = json.Unmarshal(body, &backData)
				id = backData.ID
			}
			data.ID = id
			data.Msg = projectInfo.CallbackRes
			data.Status = 0
			data.Url = *projectInfo.TargetURL
			// 将结构体转换为 JSON 格式
			jsonData, _ := json.Marshal(data)
			headers := map[string]string{
				"Content-Type": "application/json",
			}

			code, respBody, errs := sendPostRequest(callInfo.CallbackURL, []byte(jsonData), headers)
			if code != 200 || errs != nil {
				msg = fmt.Sprintf("post回调 : code = %d , 原因：%s", code, errs.Error())
				res = false
			}
			logger.Info("Response Body:", string(respBody))
		} else {
			code, respBody, errs := sendGetRequest(callInfo.CallbackURL)
			if code != 200 || errs != nil {
				msg = fmt.Sprintf("get 回调 : code = %d , 原因：%s", code, errs.Error())
				res = false
			}
			logger.Info("Response Body:", string(respBody))
		}
	}

	return res, msg
}

//回调业务方

func sendGetRequest(url string) (int, []byte, error) {
	resp, err := http.Get(url)
	if err != nil {
		return 0, nil, err
	}
	defer func() {
		_ = resp.Body.Close()
	}()

	if resp == nil {
		return 2, []byte("响应为空"), errors.New("响应请求为空")
	}

	if resp.StatusCode != 200 {
		return 4, []byte("请求失败状态码 ： "), errors.New(strconv.Itoa(resp.StatusCode))
	}

	respBody, errs := io.ReadAll(resp.Body)
	if errs != nil {
		return 5, []byte("读取body失败"), errs
	}

	return resp.StatusCode, respBody, nil
}

func sendPostRequest(url string, body []byte, headers map[string]string) (int, []byte, error) {
	req, err := http.NewRequest("POST", url, bytes.NewBuffer(body))
	if err != nil {
		return 1, []byte("创建post出错"), err
	}

	// 设置请求头
	for key, value := range headers {
		req.Header.Set(key, value)
	}

	client := &http.Client{}
	resp, errDO := client.Do(req)

	defer func() {
		_ = resp.Body.Close()
	}()

	if resp == nil {
		return 2, []byte("响应为空"), errors.New("响应请求为空")
	}
	if errDO != nil {
		return 3, []byte("http请求出错"), errDO
	}
	if resp.StatusCode != 200 {
		return 4, []byte("请求失败状态码 ： "), errors.New(strconv.Itoa(resp.StatusCode))
	}

	respBody, errRes := io.ReadAll(resp.Body)
	if errRes != nil {
		return 5, respBody, errors.New(strconv.Itoa(resp.StatusCode))
	}

	return 200, respBody, nil
}
