package console

import (
	"bufio"
	"context"
	"doc-service/domain/entity/serversitemicros/docserverproject"
	service "doc-service/domain/repository"
	service2 "doc-service/domain/service"
	"doc-service/infra/db"
	"fmt"
	"github.com/nacos-group/nacos-sdk-go/v2/common/logger"
	"gitlab.galaxy-immi.com/Backend-group/go-com/third/log"
	"os"
	"strings"
	"time"
)

func recoverPanicGetPro() {
	if r := recover(); r != nil {
		logger.Error("获取转换进度异常", r)
	}
}

// 获取转换进度
func GetConvert() {
	defer recoverPanicGetPro()
	fmt.Println("开始获取进度...")
	ctx := context.Background()
	// 启动一个协程来执行定时任务
	tickerNum := time.NewTicker(15 * time.Second)
	defer tickerNum.Stop()

	go func() {
		for {
			select {
			case <-tickerNum.C:
				i := getConvertDoData(ctx)
				if i == 0 {
					fmt.Println("没有需要获取的文件数据: " + time.Now().Format("2006-01-02 15:04:05"))
				}
			}
		}
	}()

	// 监听命令行输入
	readerGet := bufio.NewReader(os.Stdin)
	for {
		text, _ := readerGet.ReadString('\n')
		text = strings.TrimSpace(text)

		if text == "exit" {
			fmt.Println("Exiting process...")
			os.Exit(0)
		}
	}
}

func getConvertDoData(ctx context.Context) int8 {

	defer recoverPanicGetPro()
	list, err := db.GetServerSiteMicrosDB().DocServerProject.Query().Where(
		docserverproject.StatusEQ(1),
	).All(ctx)
	if err != nil {
		return 0
	}
	if len(list) <= 0 {
		return 0
	}

	for _, items := range list {
		res, msg := GetProjectInfo(ctx, items.ID)
		logger.Info("状态: %d", res)
		_, errUpdateSt := db.GetServerSiteMicrosDB().DocServerProject.UpdateOneID(items.ID).SetStatus(2).SetCallbackRes(msg).Save(ctx)
		if errUpdateSt != nil {
			log.Info("修改获取的文件进度状态失败" + errUpdateSt.Error())
			continue
		}
	}
	return int8(len(list))
}

func GetProjectInfo(c context.Context, projectId int32) (int8, string) {

	defer recoverPanicGetPro()
	projectInfo, infoErr := db.GetServerSiteMicrosDB().DocServerProject.Get(c, projectId)
	if infoErr != nil {
		return 0, infoErr.Error()
	}
	request := &service.GetTaskRequest{}
	request.ProjectId = projectId
	request.RequestDefinition = true
	request.ProjectName = projectInfo.ProjectName
	request.TaskId = projectInfo.TaskID
	//task_type和原始有区别，需要使用返回的task_id的首个元素
	splitted := strings.Split(projectInfo.TaskID, "-")
	request.TaskType = splitted[0]

	var isPrivate = true
	if projectInfo.IsPrivate == 0 {
		isPrivate = false
	}
	result, errs := service.NewConversion(isPrivate, *projectInfo.SourceURL).GetTaskProject(*request)
	if errs != nil || *result.StatusCode != 200 || *result.Body.Status != "Succeeded" {
		logger.Info(result)
		logger.Error("错误信息" + errs.Error())
		return 0, errs.Error()
	}
	service2.CreateGetProject(c, projectId, *result.Body.Status)

	return 1, "success"
}
